//-< ERMSG.CPP >-----------------------------------------------------*--------*
// Garret Object Class Library                    (c) 1992  GARRET   *     ?  *
//                                                                   *   /\|  *
//  Error handling                                                   *  /  \  *
//                           Created:      16-Mar-92   I.V. Abramow  * / [] \ *
//    Implementation         Last update:  17-Mar-92   I.V. Abramow  * GARRET *
//-------------------------------------------------------------------*--------*

#ifdef _WIN32
#include <windows.h>
#elif !defined(__MSDOS__)
#include <errno.h>
#endif

#include    <stdio.h>
#include    <string.h>
#include    <stdlib.h>

#include    "stdtp.hpp"             // Include standard type definitions
#include    "ermsg.hpp"             // Include front end library declaratiions

global  enum Err_Hook_Type  (*Msg_Warning)( const char* Format, va_list Prm );
global  enum Err_Hook_Type  (*Msg_Error  )( const char* Format, va_list Prm );
global  enum Err_Hook_Type  (*Msg_Fatal  )( const char* Format, va_list Prm );
global  enum Err_Hook_Type  (*Msg_Abzac  )( const char* Format, va_list Prm );
global  enum Err_Hook_Type  (*Msg_Trace  )( const char* Format, va_list Prm );

global  int     Warn_Lock;
global  int     Wrn_Limit;
global  int     Wrn_Count;
global  int     Err_Limit;
global  int     Err_Count;

//------------------------------------------------------------------------------
// Warning  -- Print out warning message
//
// Param :  const char* Format -- Format to use while print message
//          ...                -- Parameter list for that format
//
// Uses  :  va_start, va_end, fprintf, vfprintf, putc, exit, Error
//
//------------------------------------------------------------------------------

global  void    Warning( const char* Format, ... ){
    va_list             Arg_Ptr;
    Err_Hook_Type       Mode_Flag = Err_Hook_Cont;

    if( Warn_Lock )
        return;
    va_start( Arg_Ptr, Format );
    if( Msg_Warning != NULL )
        Mode_Flag = (*Msg_Warning)( Format, Arg_Ptr );
    if( Mode_Flag == Err_Hook_Cont ){
        fprintf( stderr, "Warning" );
        vfprintf( stderr, Format, Arg_Ptr );
        putc( '\n', stderr );
    }
    va_end( Arg_Ptr );
    if( Mode_Flag == Err_Hook_None )
        return;
    Wrn_Count += 1;
    if( Wrn_Limit && Wrn_Count >= Wrn_Limit ){
        Err_Limit = 0;                          // Prevent error limit checking
        Error( "Warning limit exceeded\n" );
        exit( EX_ERRORS );
    }
}

//------------------------------------------------------------------------------
// Error    -- Print out warning error
//
// Param :  const char* Format -- Format to use while print message
//          ...                -- Parameter list for that format
//
// Uses  :  va_start, va_end, fprintf, vsprintf, putc, exit, Error
//
//------------------------------------------------------------------------------

global  void    Error  ( const char* Format, ... ){
    va_list             Arg_Ptr;
    Err_Hook_Type       Mode_Flag = Err_Hook_Cont;

    va_start( Arg_Ptr, Format );
    if( Msg_Error != NULL )
        Mode_Flag = (*Msg_Error)( Format, Arg_Ptr );
    if( Mode_Flag == Err_Hook_Cont ){
        fprintf( stderr, "*Error*" );
        vfprintf( stderr, Format, Arg_Ptr );
        putc( '\n', stderr );
    }
    va_end( Arg_Ptr );
    if( Mode_Flag == Err_Hook_None )
        return;
    Err_Count += 1;
    if( Err_Limit && Err_Count >= Err_Limit ){
        Err_Limit = 0;                          // Prevent error limit checking
        Error( "Error limit exceeded\n" );
        exit( EX_ERRORS );
    }
}

//------------------------------------------------------------------------------
// Fatal    -- issue a fatal message and exit
//
// Param :  const char* Format -- Format to use while print message
//          ...                -- Parameter list for that format
//
// Uses  :  va_start, va_end, exit, fprintf, vfprintf, putc
//
//------------------------------------------------------------------------------

global  noret   void    Fatal  ( const char* Format, ... ){
    va_list             Arg_Ptr;
    Err_Hook_Type       Mode_Flag = Err_Hook_Cont;

    va_start( Arg_Ptr, Format );
    if( Msg_Fatal != NULL )
        Mode_Flag = (*Msg_Fatal)( Format, Arg_Ptr );
    if( Mode_Flag == Err_Hook_Cont ){
        fprintf( stderr, "*Fatal*" );
        vfprintf( stderr, Format, Arg_Ptr );
        putc( '\n', stderr );
    }
    va_end( Arg_Ptr );
    exit( EX_FATAL );
}

//------------------------------------------------------------------------------
// Abzac    -- issue an internal check fatal message and exit
//
// Param :  const char* Format -- Format to use while print message
//          ...                -- Parameter list for that format
//
// Uses  :  va_start, va_end, exit, fprintf, vsprintf, putc
//
//------------------------------------------------------------------------------

global  noret   void    Abzac  ( const char* Format, ... ){
    va_list             Arg_Ptr;
    Err_Hook_Type       Mode_Flag = Err_Hook_Cont;

    va_start( Arg_Ptr, Format );
    if( Msg_Abzac != NULL )
        Mode_Flag = (*Msg_Abzac)( Format, Arg_Ptr );
    if( Mode_Flag == Err_Hook_Cont ){
        fprintf( stderr, "*Abzac*" );
        vfprintf( stderr, Format, Arg_Ptr );
        putc( '\n', stderr );
    }
    va_end( Arg_Ptr );
    abort();
}

//------------------------------------------------------------------------------
// Trace -- Print out debugging message
//
// Param :  const char* Format -- Format to use while print message
//          ...                -- Parameter list for that format
//
// Uses  :  va_start, va_end, fprintf, vfprintf, putc
//
//------------------------------------------------------------------------------

global  void    Trace( const char* Format, ... ){
    va_list             Arg_Ptr;
    Err_Hook_Type       Mode_Flag = Err_Hook_Cont;

    va_start( Arg_Ptr, Format );
    if( Msg_Trace != NULL )
        Mode_Flag = (*Msg_Trace)( Format, Arg_Ptr );
    if( Mode_Flag == Err_Hook_Cont ){
        vfprintf( stderr, Format, Arg_Ptr );
    }
    va_end( Arg_Ptr );
}


char const*  Sys_Error()
{
#ifdef __MSDOS__
    perror( "I/O Error" ); 
    return "I/O error"; 
#elif defined(_WIN32)
    static char buf[256];
    if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
		      NULL, GetLastError(), 0,
		      buf, sizeof(buf), NULL) == 0)
    {
	sprintf(buf, "unknown error %u", GetLastError());
    }
    return buf;
#else
    return strerror(errno);
#endif
}
